@@ -507,6 +507,7 @@ static BOOL client_cli_authenticate_raw(freerdp* instance, rdp_auth_reason reaso
 	const char* domainAuth = "Domain:          ";
 	const char* pwdAuth = "Password:        ";
 	BOOL pinOnly = FALSE;
+	BOOL queryAll = FALSE;
 
 	WINPR_ASSERT(instance);
 	WINPR_ASSERT(instance->context);
@@ -519,6 +520,8 @@ static BOOL client_cli_authenticate_raw(freerdp* instance, rdp_auth_reason reaso
 			pinOnly = TRUE;
 			break;
 		case AUTH_RDSTLS:
+			queryAll = TRUE;
+			break;
 		case AUTH_TLS:
 		case AUTH_RDP:
 		case AUTH_NLA:
@@ -539,16 +542,24 @@ static BOOL client_cli_authenticate_raw(freerdp* instance, rdp_auth_reason reaso
 
 	if (!pinOnly)
 	{
-		const int rc = client_cli_read_string(instance, userAuth, *username, username);
-		if (rc < 0)
-			goto fail;
+		const char* suggest = *username;
+		if (queryAll || !suggest)
+		{
+			const int rc = client_cli_read_string(instance, userAuth, suggest, username);
+			if (rc < 0)
+				goto fail;
+		}
 	}
 
 	if (!pinOnly)
 	{
-		const int rc = client_cli_read_string(instance, domainAuth, *domain, domain);
-		if (rc < 0)
-			goto fail;
+		const char* suggest = *domain;
+		if (queryAll || !suggest)
+		{
+			const int rc = client_cli_read_string(instance, domainAuth, suggest, domain);
+			if (rc < 0)
+				goto fail;
+		}
 	}
 
 	{
@@ -1087,13 +1098,14 @@ static BOOL client_cli_get_rdsaad_access_token(freerdp* instance, const char* sc
 	if (freerdp_interruptible_get_line(instance->context, &url, &size, stdin) < 0)
 		goto cleanup;
 
-	const char* code = extract_authorization_code(url);
-	if (!code)
-		goto cleanup;
-
-	token_request =
-	    freerdp_client_get_aad_url((rdpClientContext*)instance->context,
-	                               FREERDP_CLIENT_AAD_TOKEN_REQUEST, scope, code, req_cnf);
+	{
+		const char* code = extract_authorization_code(url);
+		if (!code)
+			goto cleanup;
+		token_request =
+		    freerdp_client_get_aad_url((rdpClientContext*)instance->context,
+		                               FREERDP_CLIENT_AAD_TOKEN_REQUEST, scope, code, req_cnf);
+	}
 	if (!token_request)
 		goto cleanup;
 
@@ -1131,12 +1143,13 @@ static BOOL client_cli_get_avd_access_token(freerdp* instance, char** token)
 	if (freerdp_interruptible_get_line(instance->context, &url, &size, stdin) < 0)
 		goto cleanup;
 
-	const char* code = extract_authorization_code(url);
-	if (!code)
-		goto cleanup;
-
-	token_request = freerdp_client_get_aad_url((rdpClientContext*)instance->context,
-	                                           FREERDP_CLIENT_AAD_AVD_TOKEN_REQUEST, code);
+	{
+		const char* code = extract_authorization_code(url);
+		if (!code)
+			goto cleanup;
+		token_request = freerdp_client_get_aad_url((rdpClientContext*)instance->context,
+		                                           FREERDP_CLIENT_AAD_AVD_TOKEN_REQUEST, code);
+	}
 
 	if (!token_request)
 		goto cleanup;
@@ -1299,7 +1312,7 @@ SSIZE_T client_common_retry_dialog(freerdp* instance, const char* what, size_t c
 	}
 
 	WLog_INFO(TAG, "[%s] retry %" PRIuz "/%" PRIuz ", delaying %" PRIuz "ms before next attempt",
-	          what, current, max, delay);
+	          what, current + 1, max, delay);
 	return WINPR_ASSERTING_INT_CAST(SSIZE_T, delay);
 }
 
@@ -1364,7 +1377,7 @@ BOOL client_auto_reconnect_ex(freerdp* instance, BOOL (*window_events)(freerdp*
 	while (retry)
 	{
 		/* Quit retrying if max retries has been exceeded */
-		if ((maxRetries > 0) && (numRetries++ >= maxRetries))
+		if ((maxRetries > 0) && (numRetries >= maxRetries))
 		{
 			WLog_DBG(TAG, "AutoReconnect retries exceeded.");
 			return FALSE;
@@ -1375,6 +1388,9 @@ BOOL client_auto_reconnect_ex(freerdp* instance, BOOL (*window_events)(freerdp*
 
 		const SSIZE_T delay =
 		    IFCALLRESULT(5000, instance->RetryDialog, instance, "connection", numRetries, NULL);
+		if (delay < 0)
+			return FALSE;
+		numRetries++;
 
 		if (freerdp_reconnect(instance))
 			return TRUE;
@@ -1387,7 +1403,7 @@ BOOL client_auto_reconnect_ex(freerdp* instance, BOOL (*window_events)(freerdp*
 			default:
 				break;
 		}
-		for (UINT32 x = 0; x < delay / 10; x++)
+		for (SSIZE_T x = 0; x < delay / 10; x++)
 		{
 			if (!IFCALLRESULT(TRUE, window_events, instance))
 			{
@@ -2428,15 +2444,21 @@ static char* aad_auth_request(rdpClientContext* cctx, WINPR_ATTR_UNUSED va_list
 	const char* client_id = freerdp_settings_get_string(settings, FreeRDP_GatewayAvdClientID);
 	if (!client_id || !redirect_uri)
 		goto cleanup;
-	const char* scope = va_arg(ap, const char*);
-	if (!scope)
-		goto cleanup;
 
-	const char* ep = freerdp_utils_aad_get_wellknown_string(&cctx->context,
-	                                                        AAD_WELLKNOWN_authorization_endpoint);
+	{
+		const char* scope = va_arg(ap, const char*);
+		if (!scope)
+			goto cleanup;
+
+		{
+			const char* ep = freerdp_utils_aad_get_wellknown_string(
+			    &cctx->context, AAD_WELLKNOWN_authorization_endpoint);
+			winpr_asprintf(&url, &urllen,
+			               "%s?client_id=%s&response_type=code&scope=%s&redirect_uri=%s", ep,
+			               client_id, scope, redirect_uri);
+		}
+	}
 
-	winpr_asprintf(&url, &urllen, "%s?client_id=%s&response_type=code&scope=%s&redirect_uri=%s", ep,
-	               client_id, scope, redirect_uri);
 cleanup:
 	free(redirect_uri);
 	return url;
