@@ -260,32 +260,34 @@ static BOOL rdp_redirection_read_base64_wchar(UINT32 flag, wStream* s, UINT32* p
 	if (!*pData)
 		goto fail;
 
-	size_t rlen = utf8_len;
-	size_t wpos = 0;
-	char* saveptr = NULL;
-	char* tok = strtok_s(utf8, "\r\n", &saveptr);
-	while (tok)
 	{
-		const size_t len = strnlen(tok, rlen);
-		rlen -= len;
+		size_t rlen = utf8_len;
+		size_t wpos = 0;
+		char* saveptr = NULL;
+		char* tok = strtok_s(utf8, "\r\n", &saveptr);
+		while (tok)
+		{
+			const size_t len = strnlen(tok, rlen);
+			rlen -= len;
 
-		size_t bplen = 0;
-		BYTE* bptr = NULL;
-		crypto_base64_decode(tok, len, &bptr, &bplen);
-		if (!bptr)
+			size_t bplen = 0;
+			BYTE* bptr = NULL;
+			crypto_base64_decode(tok, len, &bptr, &bplen);
+			if (!bptr)
+				goto fail;
+			memcpy(&(*pData)[wpos], bptr, bplen);
+			wpos += bplen;
+			free(bptr);
+
+			tok = strtok_s(NULL, "\r\n", &saveptr);
+		}
+		if (wpos > UINT32_MAX)
 			goto fail;
-		memcpy(&(*pData)[wpos], bptr, bplen);
-		wpos += bplen;
-		free(bptr);
 
-		tok = strtok_s(NULL, "\r\n", &saveptr);
+		*pLength = (UINT32)wpos;
+
+		WLog_DBG(TAG, "%s:", rdp_redirection_flags_to_string(flag, buffer, sizeof(buffer)));
 	}
-	if (wpos > UINT32_MAX)
-		goto fail;
-
-	*pLength = (UINT32)wpos;
-
-	WLog_DBG(TAG, "%s:", rdp_redirection_flags_to_string(flag, buffer, sizeof(buffer)));
 
 	rc = TRUE;
 fail:
@@ -974,122 +976,128 @@ BOOL rdp_write_enhanced_security_redirection_packet(wStream* s, const rdpRedirec
 
 	Stream_Write_UINT16(s, 0);
 
-	const size_t start = Stream_GetPosition(s);
-	Stream_Write_UINT16(s, SEC_REDIRECTION_PKT);
-	const size_t lengthOffset = Stream_GetPosition(s);
-	Stream_Seek_UINT16(s); /* placeholder for length */
-
-	if (redirection->sessionID)
-		Stream_Write_UINT32(s, redirection->sessionID);
-	else
-		Stream_Write_UINT32(s, 0);
-
-	Stream_Write_UINT32(s, redirection->flags);
-
-	if (redirection->flags & LB_TARGET_NET_ADDRESS)
 	{
-		if (redir_write_string(LB_TARGET_NET_ADDRESS, s, redirection->TargetNetAddress) < 0)
-			goto fail;
-	}
+		const size_t start = Stream_GetPosition(s);
+		Stream_Write_UINT16(s, SEC_REDIRECTION_PKT);
+		const size_t lengthOffset = Stream_GetPosition(s);
+		Stream_Seek_UINT16(s); /* placeholder for length */
 
-	if (redirection->flags & LB_LOAD_BALANCE_INFO)
-	{
-		const UINT32 length = 13 + redirection->LoadBalanceInfoLength + 2;
-		if (!Stream_EnsureRemainingCapacity(s, length))
-			goto fail;
-		Stream_Write_UINT32(s, length);
-		Stream_Write(s, "Cookie: msts=", 13);
-		Stream_Write(s, redirection->LoadBalanceInfo, redirection->LoadBalanceInfoLength);
-		Stream_Write_UINT8(s, 0x0d);
-		Stream_Write_UINT8(s, 0x0a);
-	}
+		if (redirection->sessionID)
+			Stream_Write_UINT32(s, redirection->sessionID);
+		else
+			Stream_Write_UINT32(s, 0);
 
-	if (redirection->flags & LB_USERNAME)
-	{
-		if (redir_write_string(LB_USERNAME, s, redirection->Username) < 0)
-			goto fail;
-	}
+		Stream_Write_UINT32(s, redirection->flags);
 
-	if (redirection->flags & LB_DOMAIN)
-	{
-		if (redir_write_string(LB_DOMAIN, s, redirection->Domain) < 0)
-			goto fail;
-	}
-
-	if (redirection->flags & LB_PASSWORD)
-	{
-		/* Password is either UNICODE or opaque data */
-		if (!redir_write_data(LB_PASSWORD, s, redirection->PasswordLength, redirection->Password))
-			goto fail;
-	}
-
-	if (redirection->flags & LB_TARGET_FQDN)
-	{
-		if (redir_write_string(LB_TARGET_FQDN, s, redirection->TargetFQDN) < 0)
-			goto fail;
-	}
-
-	if (redirection->flags & LB_TARGET_NETBIOS_NAME)
-	{
-		if (redir_write_string(LB_TARGET_NETBIOS_NAME, s, redirection->TargetNetBiosName) < 0)
-			goto fail;
-	}
-
-	if (redirection->flags & LB_CLIENT_TSV_URL)
-	{
-		if (!redir_write_data(LB_CLIENT_TSV_URL, s, redirection->TsvUrlLength, redirection->TsvUrl))
-			goto fail;
-	}
-
-	if (redirection->flags & LB_REDIRECTION_GUID)
-	{
-		if (!redir_write_data(LB_REDIRECTION_GUID, s, redirection->RedirectionGuidLength,
-		                      redirection->RedirectionGuid))
-			goto fail;
-	}
-
-	if (redirection->flags & LB_TARGET_CERTIFICATE)
-	{
-		if (!rdp_redireciton_write_target_cert_stream(s, redirection))
-			goto fail;
-	}
-
-	if (redirection->flags & LB_TARGET_NET_ADDRESSES)
-	{
-		UINT32 length = sizeof(UINT32);
-
-		if (!Stream_EnsureRemainingCapacity(s, 2 * sizeof(UINT32)))
-			goto fail;
-
-		const size_t lstart = Stream_GetPosition(s);
-		Stream_Seek_UINT32(s); /* length of field */
-		Stream_Write_UINT32(s, redirection->TargetNetAddressesCount);
-		for (UINT32 i = 0; i < redirection->TargetNetAddressesCount; i++)
+		if (redirection->flags & LB_TARGET_NET_ADDRESS)
 		{
-			const SSIZE_T rcc =
-			    redir_write_string(LB_TARGET_NET_ADDRESSES, s, redirection->TargetNetAddresses[i]);
-			if (rcc < 0)
+			if (redir_write_string(LB_TARGET_NET_ADDRESS, s, redirection->TargetNetAddress) < 0)
 				goto fail;
-			length += (UINT32)rcc;
 		}
 
-		/* Write length field */
-		const size_t lend = Stream_GetPosition(s);
-		Stream_SetPosition(s, lstart);
-		Stream_Write_UINT32(s, length);
-		Stream_SetPosition(s, lend);
+		if (redirection->flags & LB_LOAD_BALANCE_INFO)
+		{
+			const UINT32 length = 13 + redirection->LoadBalanceInfoLength + 2;
+			if (!Stream_EnsureRemainingCapacity(s, length))
+				goto fail;
+			Stream_Write_UINT32(s, length);
+			Stream_Write(s, "Cookie: msts=", 13);
+			Stream_Write(s, redirection->LoadBalanceInfo, redirection->LoadBalanceInfoLength);
+			Stream_Write_UINT8(s, 0x0d);
+			Stream_Write_UINT8(s, 0x0a);
+		}
+
+		if (redirection->flags & LB_USERNAME)
+		{
+			if (redir_write_string(LB_USERNAME, s, redirection->Username) < 0)
+				goto fail;
+		}
+
+		if (redirection->flags & LB_DOMAIN)
+		{
+			if (redir_write_string(LB_DOMAIN, s, redirection->Domain) < 0)
+				goto fail;
+		}
+
+		if (redirection->flags & LB_PASSWORD)
+		{
+			/* Password is either UNICODE or opaque data */
+			if (!redir_write_data(LB_PASSWORD, s, redirection->PasswordLength,
+			                      redirection->Password))
+				goto fail;
+		}
+
+		if (redirection->flags & LB_TARGET_FQDN)
+		{
+			if (redir_write_string(LB_TARGET_FQDN, s, redirection->TargetFQDN) < 0)
+				goto fail;
+		}
+
+		if (redirection->flags & LB_TARGET_NETBIOS_NAME)
+		{
+			if (redir_write_string(LB_TARGET_NETBIOS_NAME, s, redirection->TargetNetBiosName) < 0)
+				goto fail;
+		}
+
+		if (redirection->flags & LB_CLIENT_TSV_URL)
+		{
+			if (!redir_write_data(LB_CLIENT_TSV_URL, s, redirection->TsvUrlLength,
+			                      redirection->TsvUrl))
+				goto fail;
+		}
+
+		if (redirection->flags & LB_REDIRECTION_GUID)
+		{
+			if (!redir_write_data(LB_REDIRECTION_GUID, s, redirection->RedirectionGuidLength,
+			                      redirection->RedirectionGuid))
+				goto fail;
+		}
+
+		if (redirection->flags & LB_TARGET_CERTIFICATE)
+		{
+			if (!rdp_redireciton_write_target_cert_stream(s, redirection))
+				goto fail;
+		}
+
+		if (redirection->flags & LB_TARGET_NET_ADDRESSES)
+		{
+			UINT32 length = sizeof(UINT32);
+
+			if (!Stream_EnsureRemainingCapacity(s, 2 * sizeof(UINT32)))
+				goto fail;
+
+			const size_t lstart = Stream_GetPosition(s);
+			Stream_Seek_UINT32(s); /* length of field */
+			Stream_Write_UINT32(s, redirection->TargetNetAddressesCount);
+			for (UINT32 i = 0; i < redirection->TargetNetAddressesCount; i++)
+			{
+				const SSIZE_T rcc = redir_write_string(LB_TARGET_NET_ADDRESSES, s,
+				                                       redirection->TargetNetAddresses[i]);
+				if (rcc < 0)
+					goto fail;
+				length += (UINT32)rcc;
+			}
+
+			/* Write length field */
+			const size_t lend = Stream_GetPosition(s);
+			Stream_SetPosition(s, lstart);
+			Stream_Write_UINT32(s, length);
+			Stream_SetPosition(s, lend);
+		}
+
+		/* Padding 8 bytes */
+		if (!Stream_EnsureRemainingCapacity(s, 8))
+			goto fail;
+		Stream_Zero(s, 8);
+
+		{
+			const size_t end = Stream_GetPosition(s);
+			Stream_SetPosition(s, lengthOffset);
+			Stream_Write_UINT16(s, (UINT16)(end - start));
+			Stream_SetPosition(s, end);
+		}
 	}
 
-	/* Padding 8 bytes */
-	if (!Stream_EnsureRemainingCapacity(s, 8))
-		goto fail;
-	Stream_Zero(s, 8);
-
-	const size_t end = Stream_GetPosition(s);
-	Stream_SetPosition(s, lengthOffset);
-	Stream_Write_UINT16(s, (UINT16)(end - start));
-	Stream_SetPosition(s, end);
-
 	rc = TRUE;
 fail:
 	return rc;
